using System;
using System.ComponentModel;
using System.Linq;
using System.Web.Mvc;
using Mayando.Web.Extensions;
using Mayando.Web.Infrastructure;
using Mayando.Web.Models;
using Mayando.Web.ViewModels;

namespace Mayando.Web.Controllers
{
    [Description("Handles actions that have to do with galleries.")]
    public class GalleriesController : SiteControllerBase
    {
        #region Constants

        public const string ControllerName = "galleries";

        #endregion

        #region Actions

        [Description("Shows all galleries.")]
        public ActionResult Index()
        {
            return ViewFor(ViewName.Index, r =>
                {
                    var galleries = r.GetGalleriesWithTagsAndCoverPhoto();
                    return galleries.Select(g => new GalleryViewModel(g, new PhotosViewModel(this.GetPhotosForGallery(r, g, 0, 0)))).ToList();
                });
        }

        [Description("Shows a single gallery.")]
        public ActionResult Details([Description("The gallery id.")]int id, [Description("The number of items to show.")]int? count, [Description("The page number to show.")]int? page)
        {
            return ViewForGallery(r => r.GetGalleryWithTagsAndCoverPhotoById(id), count, page);
        }

        [Description("Shows the gallery with a certain title.")]
        public ActionResult Titled([Description("The slug (URL-friendly version) of the gallery's title.")]string id, [Description("The number of items to show.")]int? count, [Description("The page number to show.")]int? page)
        {
            return ViewForGallery(r => r.GetGalleryWithTagsAndCoverPhotoBySlug(id), count, page);
        }

        [Description("Allows the user to create a new gallery.")]
        [AuthorizeAdministrator]
        public ActionResult Create([Description("The photo id to show as the cover for the gallery.")]int? coverPhotoId)
        {
            this.StorePhotoListInViewData(coverPhotoId, true);
            return ViewForCreate<Gallery>((r, g) =>
            {
                g.Sequence = r.GetMaxGallerySequence() + 1;
            });
        }

        [AuthorizeAdministrator]
        [AcceptVerbs(HttpVerbs.Post)]
        [ValidateInput(false)]
        public ActionResult Create([Bind(Exclude = EntityPropertyNameId)]Gallery gallery, int? coverPhotoId, string tagList)
        {
            var tags = Converter.ToTags(tagList);
            this.StorePhotoListInViewData(coverPhotoId, true);
            return PerformSave(ViewName.Create, gallery, r => r.CreateGallery(gallery, coverPhotoId, tags));
        }

        [Description("Allows the user to edit a gallery.")]
        [AuthorizeAdministrator]
        public ActionResult Edit([Description("The gallery id.")]int id)
        {
            return ViewFor(ViewName.Edit, r =>
            {
                var gallery = r.GetGalleryWithTagsAndCoverPhotoById(id);
                int? selectedPhotoId = null;
                if (gallery.CoverPhoto != null)
                {
                    selectedPhotoId = gallery.CoverPhoto.Id;
                }
                this.StorePhotoListInViewData(selectedPhotoId, true);
                return gallery;
            });
        }

        [AuthorizeAdministrator]
        [AcceptVerbs(HttpVerbs.Post)]
        [ValidateInput(false)]
        public ActionResult Edit(Gallery gallery, int? coverPhotoId, string tagList)
        {
            var tags = Converter.ToTags(tagList);
            this.StorePhotoListInViewData(coverPhotoId, true);
            return PerformSave(ViewName.Edit, gallery, r => r.SaveGallery(gallery, coverPhotoId, tags));
        }

        [Description("Allows the user to delete a gallery.")]
        [AuthorizeAdministrator]
        public ActionResult Delete([Description("The gallery id.")]int id)
        {
            return ViewForDelete("Gallery", g => g.Title, r => r.GetGalleryById(id));
        }

        [AuthorizeAdministrator]
        [ActionName(ActionNameDelete)]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult DeletePost(int id)
        {
            return PerformDelete(r => r.DeleteGallery(id));
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Move(int id, Direction direction)
        {
            using (var repository = GetRepository(true))
            {
                var galleries = repository.GetGalleries();
                var currentItem = (ISupportSequence)galleries.FirstOrDefault(m => m.Id == id);
                this.MoveInSequence(direction, galleries.Cast<ISupportSequence>(), currentItem);
                repository.CommitChanges();
                return RedirectToAction(ActionName.Index);
            }
        }

        #endregion

        #region Helper Methods

        private ActionResult ViewForGallery(Func<MayandoRepository, Gallery> gallerySelector, int? count, int? page)
        {
            using (var repository = GetRepository())
            {
                var gallery = gallerySelector(repository);
                if (gallery == null)
                {
                    return View(ViewName.NotFound);
                }
                var photos = this.GetPhotosForGallery(repository, gallery, GetActualCount(count), GetActualPage(page));
                return View(ViewName.Details, new GalleryViewModel(gallery, new PhotosViewModel(photos, NavigationContextType.Gallery, gallery.Slug)));
            }
        }

        #endregion
    }
}