using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using JelleDruyts.Web.Mvc;
using Mayando.ProviderModel;
using Mayando.Web.Infrastructure;
using Mayando.Web.Models;
using Mayando.Web.Properties;
using Mayando.Web.ViewModels;

namespace Mayando.Web.Controllers
{
    [Description("Handles actions that have to do with settings.")]
    [AuthorizeAdministrator]
    public class SettingsController : SiteControllerBase
    {
        #region Constants

        public const string ControllerName = "settings";

        #endregion

        #region Actions

        [Description("Allows the user to edit the application settings.")]
        public ActionResult Index()
        {
            using (var repository = GetRepository())
            {
                var settings = repository.GetSettings(SettingsScope.Application);
                var settingsViewModel = GetSettingsViewModel(settings, this.ControllerContext.HttpContext);
                return View(settingsViewModel);
            }
        }

        [AcceptVerbs(HttpVerbs.Post)]
        [ValidateInput(false)]
        public ActionResult Edit(IDictionary<string, string> settings)
        {
            using (var repository = GetRepository(true))
            {
                repository.EnsureSettings(SettingsScope.Application, ApplicationSettings.GetSettingDefinitions());
                repository.SaveSettingValues(SettingsScope.Application, settings);
                repository.CommitChanges();
                SetPageFlash("The application settings were saved.");
                return RedirectToAction(ActionName.Index, AdminController.ControllerName);
            }
        }

        #endregion

        #region Helper Methods

        internal static IEnumerable<SettingViewModel> GetSettingsViewModel(IEnumerable<Setting> settings, HttpContextBase httpContext)
        {
            var settingsModel = new List<SettingViewModel>();
            foreach (var setting in settings)
            {
                List<SelectListItem> allowedValues = null;
                if (setting.SettingType == SettingType.TimeZone)
                {
                    allowedValues = new List<SelectListItem>();
                    var allTimeZones = new List<TimeZoneInfo>(TimeZoneInfo.GetSystemTimeZones());
                    // Always add the UTC time zone since that isn't always installed in the system.
                    allTimeZones.Add(TimeZoneInfo.Utc);
                    foreach (var timeZone in allTimeZones.OrderBy(t => t.BaseUtcOffset))
                    {
                        var selected = string.Equals(setting.Value, timeZone.Id, StringComparison.OrdinalIgnoreCase);
                        allowedValues.Add(new SelectListItem { Value = timeZone.Id, Text = timeZone.DisplayName, Selected = selected });
                    }
                }
                else if (setting.SettingType == SettingType.Theme)
                {
                    var themes = ThemedWebFormViewEngine.GetAvailableThemes(httpContext);
                    allowedValues = new List<SelectListItem>();
                    allowedValues.Add(new SelectListItem { Value = string.Empty, Text = Resources.SelectListItemNone });
                    foreach (var theme in themes)
                    {
                        var selected = string.Equals(setting.Value, theme, StringComparison.OrdinalIgnoreCase);
                        allowedValues.Add(new SelectListItem { Value = theme, Text = theme, Selected = selected });
                    }
                }
                settingsModel.Add(new SettingViewModel(setting, allowedValues));
            }
            return settingsModel;
        }

        #endregion
    }
}