﻿using System;
using System.Collections.Generic;
using Mayando.Web.Models;
using System.Globalization;

namespace Mayando.Web.Infrastructure
{
    /// <summary>
    /// Allows logging of events.
    /// </summary>
    internal static class Logger
    {
        #region Log

        /// <summary>
        /// Logs an event.
        /// </summary>
        /// <param name="level">The log level.</param>
        /// <param name="message">The log message.</param>
        public static void Log(LogLevel level, string message)
        {
            Log(level, message, null);
        }

        /// <summary>
        /// Logs an exception.
        /// </summary>
        /// <param name="exc">The exception.</param>
        public static void LogException(Exception exc)
        {
            Log(LogLevel.Error, exc.Message, exc.ToString());
        }

        /// <summary>
        /// Logs an event.
        /// </summary>
        /// <param name="level">The log level.</param>
        /// <param name="message">The log message.</param>
        /// <param name="detail">The log detail.</param>
        public static void Log(LogLevel level, string message, string detail)
        {
            var entry = new Log
            {
                Time = DateTimeOffset.UtcNow,
                LogLevel = level,
                Message = message,
                Detail = detail
            };
            Log(entry);
        }

        /// <summary>
        /// Logs an event.
        /// </summary>
        /// <param name="entry">The log entry.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private static void Log(Log entry)
        {
            try
            {
                using (var repository = MayandoRepositoryFactory.CreateRepository(true, true))
                {
                    repository.CreateLog(entry);

                    // Send an error notification mail if necessary.
                    if (entry.LogLevel == LogLevel.Error)
                    {
                        var settingValues = repository.GetSettingValues(SettingsScope.Application);
                        var settings = new ApplicationSettings(settingValues);
                        if (settings.NotifyOnErrors)
                        {
                            var subject = string.Format(CultureInfo.CurrentCulture, "An error occurred on your \"{0}\" {1} website.", settings.Title, SiteData.GlobalApplicationName);
                            var body = string.Format(CultureInfo.CurrentCulture, "An error occurred on your \"{1}\" {2} website. See the event log on your website for more information.{0}{0}{3}", Environment.NewLine, settings.Title, SiteData.GlobalApplicationName, entry.Message);
                            if (!string.IsNullOrEmpty(entry.Detail))
                            {
                                body += string.Format(CultureInfo.CurrentCulture, "{0}{0}{1}", Environment.NewLine, entry.Detail);
                            }
                            Mailer.SendNotificationMail(settings, subject, body, false, true, true);
                        }
                    }

                    repository.CommitChanges();
                }
            }
            catch (Exception)
            {
                // Ignore any exception during logging.
            }
        }

        #endregion

        #region GetLogs

        /// <summary>
        /// Gets all the logs.
        /// </summary>
        /// <returns>All log entries.</returns>
        public static IEnumerable<Log> GetLogs(int count, int page)
        {
            using (var repository = MayandoRepositoryFactory.CreateRepository())
            {
                return repository.GetLogs(count, page);
            }
        }

        #endregion

        #region Clear

        /// <summary>
        /// Clears the event log entries older than a certain age.
        /// </summary>
        /// <param name="minAge">The minimum days the entry should be old to be deleted.</param>
        public static void Clear(int minAge)
        {
            using (var repository = MayandoRepositoryFactory.CreateRepository(true, true))
            {
                repository.DeleteLogs(minAge);
                repository.CommitChanges();
            }
        }

        #endregion

        #region GetHighestLogLevel

        public static LogLevel? GetHighestLogLevel(DateTimeOffset? minTime)
        {
            using (var repository = MayandoRepositoryFactory.CreateRepository())
            {
                return repository.GetHighestLogLevel(minTime);
            }
        }

        #endregion
    }
}